package gp_dbc.config;

import bspkrs.bspkrscore.fml.Reference;
import cpw.mods.fml.client.event.ConfigChangedEvent;
import cpw.mods.fml.common.eventhandler.SubscribeEvent;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

import gp_dbc.Main;
import gp_dbc.utils.Pos;
import net.minecraftforge.common.config.Configuration;
import net.minecraftforge.common.config.Property;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class ConfigManager {

    private static Configuration config;
    public static boolean updateCheck = true;

    public static final String CATEGORY_GP = "Galactic Patrol";
    public static final String CATEGORY_GPE = "Galactic Patrol Error";
    public static final String CATEGORY_GPS = "Galactic Patrol Success";

    public static String DIR_SAVE = "./" + Main.MODID + "/";

    public static byte MIN_ALIGN_TO_JOIN = 33;
    public static String ERROR_MIN_ALIGN_TO_JOIN = "Alignment too low";
    public static byte MAX_ALIGN_TO_JOIN = 100;
    public static String ERROR_MAX_ALIGN_TO_JOIN = "Alignment too high";

    public static String SUCCESS_JOIN_GALACTIC_PATROL = "You have joined Galactic Patrol";
    public static String ERROR_JOIN_GALACTIC_PATROL = "You are already in Galactic Patrol";

    public static String SUCCESS_LEAVE_GALACTIC_PATROL = "You have left Galactic Patrol";
    public static String ERROR_LEAVE_GALACTIC_PATROL = "You aren't in Galactic Patrol";

    public static String SUCCESS_ADD_CELL = "You have added a new cell";
    public static String ERROR_ADD_CELL = "There is already a cell here";

    public static String ERROR_DEFINITION_EXIT_POS = "There isn't exit position";
    public static String ERROR_INVALID_VALUE_EXIT_POS = "Invalid value in exit position";
    public static String ERROR_UNAVAILABLE_CELL = "There aren't cell available";
    public static String ERROR_ALREADY_IN_JAIL = "Player already in jail";
    public static String ERROR_NOT_IN_JAIL = "Player not in jail";
    public static String SUCCESS_IN_JAIL = "Player put in jail for ";
    public static String MSG_INMATE_GOING_TO_JAIL = "You are in jail for ";
    public static int FORMAT_TIME_JAIL = 4;

    static public int SEC_TO_BE_IN_JAIL = 120; // second
    static public String POS_EXIT_JAIL = ""; //"-247 66 275 20";
    static public Pos pos_exit_jail;

    public static void init(String configDir) {
        if (config != null)
            return ;

        File path = new File(configDir + "/" + Main.MODID + ".cfg");

        config = new Configuration(path);
        loadConfiguration();
    }

    private static void loadConfiguration() {
        updateCheck = config.getBoolean("Check For Updates", Configuration.CATEGORY_GENERAL, true, "Allow this mod to check for updates.");

        List<String> order = new ArrayList<String>();

        DIR_SAVE = addVariableConfig(order, CATEGORY_GP, "DIR_SAVE",
                "Directory where data is saved. [default: " + "./" + Main.MODID + "/" + " ]", "./" + Main.MODID + "/").getString();

        MIN_ALIGN_TO_JOIN = (byte)addVariableConfig(order, CATEGORY_GP, "MIN_ALIGN_TO_JOIN",
                "Minimum alignment to join galactic patrol. [default: 33]", "33").getInt();

        MAX_ALIGN_TO_JOIN = (byte)addVariableConfig(order, CATEGORY_GP, "MAX_ALIGN_TO_JOIN",
                "Maximum alignment to join galactic patrol. [default: 100]", "100").getInt();

        POS_EXIT_JAIL = addVariableConfig(order, CATEGORY_GP, "POS_EXIT_JAIL",
                "Spawn point when a jailer leaves the prison. It's mandatory to put a player in jail. \"<x> <y> <z> <dimension id>\" [default: \"\"]", "").getString();
        try {
            String[] split = ConfigManager.POS_EXIT_JAIL.split(" ");
            pos_exit_jail = new Pos(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]), Integer.parseInt(split[3]));
        } catch (Exception e) {
            System.out.println(e.getMessage());
            pos_exit_jail = null;
        }

        config.setCategoryPropertyOrder(CATEGORY_GP, order);

        loadConfigurationError();
        loadConfigurationSuccess();

        if (config.hasChanged())
            config.save();
    }
    
    private static Property addVariableConfig(List<String> order, String category, String key, String comment, String defaultValue) {
        Property prop;
        prop = config.get(category, key, defaultValue);
        prop.comment = comment;
        order.add(prop.getName());
        return prop;
    }

    @SubscribeEvent
    public void onConfigurationChangeEvent(ConfigChangedEvent.OnConfigChangedEvent event) {
        if (event.modID.equalsIgnoreCase(Main.MODID))
            loadConfiguration();
    }

    public static Configuration getConfiguration() {return config;}

    private static void loadConfigurationError() {
        List<String> orderError = new ArrayList<String>();

        ERROR_MIN_ALIGN_TO_JOIN = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_MIN_ALIGN_TO_JOIN",
                "Error text in chat when a player tries to join without enough alignment. [default: \"Alignment too low\"]", "Alignment too low").getString();

        ERROR_MAX_ALIGN_TO_JOIN = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_MAX_ALIGN_TO_JOIN",
                "Error text in chat when a player try to join with too much alignment. [default: \"Alignment too high\"]", "Alignment too high").getString();

        ERROR_JOIN_GALACTIC_PATROL = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_JOIN_GALACTIC_PATROL",
                "Error text when a player tries to join Galactic Patrol and already in. [default: \"You are already in Galactic Patrol\"]", "You are already in Galactic Patrol").getString();

        ERROR_LEAVE_GALACTIC_PATROL = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_LEAVE_GALACTIC_PATROL",
                "Error text when a player tries to leave Galactic Patrol and aren't in. [default: \"You aren't in Galactic Patrol\"]", "You aren't in Galactic Patrol").getString();

        ERROR_ADD_CELL = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_ADD_CELL",
                "Error text when a player tries to add a new cell. [default: \"There isn't exit position\"]", "There isn't exit position").getString();

        ERROR_DEFINITION_EXIT_POS = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_DEFINITION_EXIT_POS",
                "Error text when a player goes to jail but there isn't exit position. [default: \"There isn't exit position\"]", "There isn't exit position").getString();

        ERROR_INVALID_VALUE_EXIT_POS = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_INVALID_VALUE_EXIT_POS",
                "Error text when a player goes to jail but the exit position is invalid. [default: \"Invalid value in exit position\"]", "Invalid value in exit position").getString();

        ERROR_UNAVAILABLE_CELL = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_UNAVAILABLE_CELL",
                "Error text when a player cannot go to jail because all the cells are full or no cell is defined. [default: \"There aren't cell available\"]", "There aren't cell available").getString();

        ERROR_ALREADY_IN_JAIL = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_ALREADY_IN_JAIL",
                "Error text when a player goes to jail but he is already in. [default: \"Player already in jail\"]", "Player already in jail").getString();

        ERROR_NOT_IN_JAIL = addVariableConfig(orderError, CATEGORY_GPE, "ERROR_NOT_IN_JAIL",
                "Error text when a inmate is release but he's already free. [default: \"There is already a cell here\"]", "There is already a cell here").getString();

        config.setCategoryPropertyOrder(CATEGORY_GPE, orderError);
    }

    private static void loadConfigurationSuccess() {
        List<String> orderSuccess = new ArrayList<String>();

        SUCCESS_JOIN_GALACTIC_PATROL = addVariableConfig(orderSuccess, CATEGORY_GPS, "SUCCESS_JOIN_GALACTIC_PATROL",
                "Success text when a player join Galactic Patrol. [default: \"You joined Galactic Patrol\"]", "You joined Galactic Patrol").getString();

        SUCCESS_LEAVE_GALACTIC_PATROL = addVariableConfig(orderSuccess, CATEGORY_GPS, "SUCCESS_LEAVE_GALACTIC_PATROL",
                "Success text when a player left Galactic Patrol. [default: \"You have left Galactic Patrol\"]", "You have left Galactic Patrol").getString();

        SUCCESS_ADD_CELL = addVariableConfig(orderSuccess, CATEGORY_GPS, "SUCCESS_ADD_CELL",
                "Success text when a player adds new cell. [default: \"You have added a new cell\"]", "You have added a new cell").getString();

        SUCCESS_IN_JAIL = addVariableConfig(orderSuccess, CATEGORY_GPS, "SUCCESS_IN_JAIL",
                "Success text when a player put a other in jail. [default: \"Player put in jail for <Time>\"]", "Player put in jail for ").getString();

        MSG_INMATE_GOING_TO_JAIL = addVariableConfig(orderSuccess, CATEGORY_GPS, "MSG_INMATE_GOING_TO_JAIL",
                "Text when you are going to jail. [default: \"You are in jail for <Time>\"]", "You are in jail for ").getString();

        FORMAT_TIME_JAIL = addVariableConfig(orderSuccess, CATEGORY_GPS, "FORMAT_TIME_JAIL",
                "Format of the time print when a player going to jail; 1=dd/MM/yyyy - HH:mm:ss; 2=dd/MM - HH:mm:ss; 3=dd - HH:mm:ss; 4=HH:mm:ss. [default: 4]", "4").getInt();

        config.setCategoryPropertyOrder(CATEGORY_GPS, orderSuccess);
    }
}
