package gp_dbc.system;

import gp_dbc.utils.Pos;
import net.minecraft.entity.player.EntityPlayer;

import java.util.ArrayList;
import java.util.List;

public class Cell {

    private Pos pos;
    private int place;
    private List<Inmate> inmates;

    public Cell(int x, int y, int z) {
        this(new Pos(x, y, z), 1);
    }

    public Cell(int x, int y, int z, int dimension) {
        this(new Pos(x, y, z, dimension), 1);
    }

    public Cell(int x, int y, int z, int dimension, int place) {
        this(new Pos(x, y, z, dimension), place);
    }

    public Cell(Pos pos, int place) {
        this.pos = pos;
        this.place = place;
        this.inmates = new ArrayList<>();
    }

    public Inmate addJailer(EntityPlayer player, long sec) {
        return addJailer(new Inmate(this, player.getUniqueID(), sec));
    }

    public Inmate addJailer(Inmate inmate) {
        inmates.add(inmate);
        return inmate;
    }

    public boolean removeJailer(EntityPlayer player, long sec) {
        return removeJailer(new Inmate(this, player.getUniqueID(), sec));
    }

    public boolean removeJailer(Inmate inmate) {
        return inmates.remove(inmate);
    }

    public boolean isFullCell() {
        return inmates.size() >= place;
    }

    public Pos getPos() { return pos; }
    public int getPlace() { return place; }
    public void setPlace(int place) { this.place = place; }
    public List<Inmate> getJailers() { return inmates; }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;

        if (!(obj instanceof Cell))
            return false;

        Cell objCell = (Cell)obj;
        return pos.equals(objCell.getPos());
    }

    public boolean equals(int x, int y, int z, int dimension) {
        return pos.x == x && pos.y == y && pos.z == z && pos.dimension == dimension;
    }

}
