package gp_dbc.system;

import gp_dbc.config.ConfigManager;
import gp_dbc.utils.Data;
import gp_dbc.utils.Pos;
import gp_dbc.utils.TimeUtils;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.MathHelper;
import org.lwjgl.Sys;

import java.text.SimpleDateFormat;
import java.util.*;

public class Jail {

    private List<Cell> cells;
    private List<Inmate> inmates;
    private List<String> waitingToBeFree; // UUID
    private List<String> inmatesToBe; // UUID

    public Jail() {
        try {
            cells = listStringToListCell(Data.loadDataString("cells"));
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }

        try {
            inmates = listStringToListJailer(Data.loadDataString("jailers"));
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }

        waitingToBeFree = Data.loadDataString("waitingToBeFree");

    }

    public boolean addCell(int x, int y, int z, int dimension) {
        if (cells.contains(new Cell(x, y , z , dimension)))
            return false;

        cells.add(new Cell(x, y, z, dimension));
        Data.saveDataString(listCellToListString(), "cells");

        return true;
    }

    public boolean removeCell(int x, int y, int z, int dimension) {
        if (!cells.contains(new Cell(x, y , z , dimension)))
            return false;

        boolean removed = true;
        for (int i = 0; i < cells.size(); i++)
            if (cells.get(i).equals(x, y, z, dimension)) {
                removed = cells.remove(i) != null;
                break;
            }
        Data.saveDataString(listCellToListString(), "cells");

        return removed;
    }

    public boolean thereIsExitPos(ICommandSender chat, boolean chatMsg) {
        String[] split = ConfigManager.POS_EXIT_JAIL.split(" ");
        if (split.length != 4) {
            if (chatMsg)
                chat.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + ConfigManager.ERROR_DEFINITION_EXIT_POS));
            return false;
        }
        else {
            try {
                Integer.parseInt(split[0]); Integer.parseInt(split[1]); Integer.parseInt(split[2]); Integer.parseInt(split[3]);
            } catch (Exception e) {
                if (chatMsg)
                    chat.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + ConfigManager.ERROR_INVALID_VALUE_EXIT_POS));
                return false;
            }
        }
        return true;
    }

    public Cell putPlayerInJail(ICommandSender sender, EntityPlayer jailer, long sec, boolean chatMsg) {

        if (!thereIsExitPos(sender, chatMsg))
            return null;

        Cell cell = null;
        try {
            cell = getRandomCell();
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }

        if (cell == null) {
            if (chatMsg)
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + ConfigManager.ERROR_UNAVAILABLE_CELL));
            return null;
        }
        if (inmates.contains(new Inmate(jailer.getUniqueID()))) {
            if (chatMsg)
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + ConfigManager.ERROR_ALREADY_IN_JAIL));
            return null;
        }

        String timeString = "";
        if (ConfigManager.FORMAT_TIME_JAIL == 1)
            timeString = TimeUtils.getTimeYears(sec);
        else if (ConfigManager.FORMAT_TIME_JAIL == 2)
            timeString = TimeUtils.getTimeMonths(sec);
        else if (ConfigManager.FORMAT_TIME_JAIL == 3)
            timeString = TimeUtils.getTimeDays(sec);
        else if (ConfigManager.FORMAT_TIME_JAIL == 4)
            timeString = TimeUtils.getTimeHours(sec);

        if (ConfigManager.SUCCESS_IN_JAIL.length() > 0)
            sender.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + ConfigManager.SUCCESS_IN_JAIL + timeString));
        if (ConfigManager.MSG_INMATE_GOING_TO_JAIL.length() > 0)
            jailer.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + ConfigManager.MSG_INMATE_GOING_TO_JAIL + timeString));

        inmates.add(new Inmate(cell, jailer.getUniqueID(), sec));
        Data.saveDataString(listJailerToListString(), "jailers");

        Pos pos = cell.getPos();
        if (jailer.dimension != pos.dimension)
            jailer.travelToDimension(pos.dimension);
        jailer.setPositionAndUpdate(pos.x + 0.5, pos.y + 1, pos.z + 0.5);
        return cell;
    }

    public int removePlayerFromJail(ICommandSender sender, EntityPlayer player, boolean chatMsg) {

        if (removePlayerFromJail(sender, player.getUniqueID(), chatMsg) != 0)
            return 1;

        waitingToBeFree.remove(player.getUniqueID().toString());
        Data.saveDataString(waitingToBeFree, "waitingToBeFree");

        tpOutOfJail(player);

        return 0;
    }

    public int removePlayerFromJail(ICommandSender sender, UUID uuid, boolean chatMsg) {

        if (!thereIsExitPos(sender, chatMsg))
            return 1;

        boolean removed = false;
        for (int i = 0; i < inmates.size(); i++)
            if (inmates.get(i).equals(uuid)) {
                inmates.get(i).getCell().removeJailer(inmates.get(i));
                inmates.remove(inmates.get(i));
                removed = true;
                break;
            }

        if (!removed) {
            if (chatMsg)
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + ConfigManager.ERROR_NOT_IN_JAIL));
            return 2;
        }

        Data.saveDataString(listJailerToListString(), "jailers");
        waitingToBeFree.add(uuid.toString());
        Data.saveDataString(waitingToBeFree, "waitingToBeFree");


        return 0;
    }

    public int handcuffedPlayer(ICommandSender sender, EntityPlayer player, boolean chatMsg) {

        if (inmatesToBe.contains(player.getUniqueID().toString())) {
            sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "He's already handcuffed"));
            return 1;
        }

        if (inmates.contains(new Inmate(player.getUniqueID()))) {
            sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "He's already in jail"));
            return 2;
        }

        sender.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + "You handcuffed him, He'll go to jail in " ));
        player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "You are handcuffed, you'll go to jail in " ));
        inmatesToBe.add(player.getUniqueID().toString());

        return 0;
    }

    public void tpOutOfJail(EntityPlayer player) {
        Pos pos = ConfigManager.pos_exit_jail;
        if (player.dimension != pos.dimension)
            player.travelToDimension(pos.dimension);
        player.setPositionAndUpdate(pos.x + 0.5, pos.y, pos.z + 0.5);
    }

    public Cell getRandomCell() throws Exception {
        if (cells.size() == 0)
            return null;

        List<Cell> availableCells = new ArrayList<>();

        for (Cell cell : cells)
            if (cell.getJailers().size() < cell.getPlace())
                availableCells.add(cell);

        if (availableCells.size() == 0)
            return null;

        int index_cell = MathHelper.getRandomIntegerInRange(new Random(), 0, availableCells.size() - 1);
        Cell cell = availableCells.get(index_cell);
        if (cell == null)
            throw new Exception("[gp_dbc.system.Jail getRandomCellPos()] Null Cell not normal");

        return cell;
    }

    private List<Cell> listStringToListCell(List<String> cellsStr) throws Exception {

        cells = new ArrayList<>();
        for (String s : cellsStr) {

            String[] split = s.split("~");
            if (split.length != 4)
                throw new Exception("Error loading " + ConfigManager.DIR_SAVE + "cells.txt");

            try {
                cells.add(new Cell(Integer.parseInt(split[0]), Integer.parseInt(split[1]), Integer.parseInt(split[2]), Integer.parseInt(split[3])));
            } catch (Exception e) {
                System.out.println(e);
                throw e;
            }
        }

        return cells;
    }

    private List<String> listCellToListString() {

        List<String> cellsStr = new ArrayList<>();
        for (Cell cell : cells)
            cellsStr.add(cell.getPos().x + "~" + cell.getPos().y + "~" + cell.getPos().z + "~" + cell.getPos().dimension);

        return cellsStr;
    }

    public boolean existCell(int x, int y, int z) {
        return existCell(x, y, z, 0);
    }

    public boolean existCell(int x, int y, int z, int dimension) {
        return cells.contains(new Cell(x, y , z , dimension));
    }

    public List<Inmate> getJailers() { return inmates; }

    private List<Inmate> listStringToListJailer(List<String> cellsStr) throws Exception {

        inmates = new ArrayList<>();
        for (String s : cellsStr) {

            String[] split = s.split("~");
            if (split.length != 7)
                throw new Exception("Error loading " + ConfigManager.DIR_SAVE + "jailers.txt");

            try {
                inmates.add(new Inmate(new Cell(Integer.parseInt(split[3]), Integer.parseInt(split[4]), Integer.parseInt(split[5]), Integer.parseInt(split[6])),
                                        UUID.fromString(split[0]), Long.parseLong(split[1]), Long.parseLong(split[2])));
            } catch (Exception e) {
                System.out.println(e);
                throw e;
            }
        }

        return inmates;
    }

    private List<String> listJailerToListString() {

        List<String> cellsStr = new ArrayList<>();
        for (Inmate inmate : inmates)
            cellsStr.add(inmate.getUuid() + "~" + inmate.getSecInJail() + "~" + inmate.getTimeStartJail()
                    + "~" + inmate.getCell().getPos().x + "~" + inmate.getCell().getPos().y + "~" + inmate.getCell().getPos().z + "~" + inmate.getCell().getPos().dimension);

        return cellsStr;
    }

    public boolean isFree(UUID uuid) {
        return waitingToBeFree.contains(uuid.toString());
    }

    public boolean freePlayer(EntityPlayer player) {
        if (!isFree(player.getUniqueID()))
            return false;

        if (!thereIsExitPos(player, true))
            return false;

        tpOutOfJail(player);
        waitingToBeFree.remove(player.getUniqueID().toString());

        return true;
    }

}
